/*
    Digital circuits emulation library

    Copyright (c) 2004 Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef DCE_H_
#define DCE_H_

#include <assert.h>

typedef unsigned char DBit;

class DChip;

class DLine
{
public:
    DLine() {
        bit_ = &Unconnected;
        assigned_ = false;
        connected_ = false;

        numOfLines++;
    }

    DLine( const DLine & line ) {
        bit_ = line.bit_;
        assigned_ = false;
        connected_ = true;

        numOfLines++;
    }

    ~DLine() {
    }

    void connectTo( const DLine & line ) {
        assert( ! assigned_ );
        assert( line.connected_ );

        bit_ = line.bit_;
        connected_ = true;

        numOfLinesConnected++;
    }

    void connectTo( DChip & chip, unsigned line );

    void assign( DBit & bit ) {
        assert( ! assigned_ );

        bit_ = &bit;

        assigned_ = true;
        connected_ = true;

        numOfLinesAssigned++;
    }

    DBit state() const {
        return *bit_;
    }

    bool isConnected() const {
        return bit_ != &Unconnected;
    }

    DBit * connection() {
        return bit_;
    }

    static DBit Unconnected;
    static DBit Power;
    static DBit Ground;
    static DBit High;
    static DBit Low;

    static unsigned numOfLines;
    static unsigned numOfLinesAssigned;
    static unsigned numOfLinesConnected;

private:
    DLine & operator = ( const DLine & );

    DBit *  bit_;
    bool assigned_;
    bool connected_;
};

class DChip
{
public:
    DChip( unsigned numOfLines ) {
        assert( numOfLines > 0 );

        line_count_ = numOfLines;
        line_ = new DLine [ numOfLines ];

        numOfChips++;
        numOfChipLines += numOfLines;
    }

    virtual ~DChip() {
        delete [] line_;
    }

    virtual void update() = 0;

    DLine & getLine( unsigned index ) {
        assert( index > 0 );
        assert( index <= line_count_ );

        index--;

        return line_[index];
    }

    void connectLine( unsigned pin, DLine & otherLine ) {
        assert( pin > 0 );
        assert( pin <= line_count_ );

        line_[pin-1].connectTo( otherLine );
    }

    void connectLine( unsigned pin, DChip & otherChip, unsigned otherPin ) {
        assert( otherPin > 0 );
        assert( otherPin <= otherChip.line_count_ );

        connectLine( pin, otherChip.line_[otherPin-1] );
    }

    static unsigned numOfChips;
    static unsigned numOfChipLines;

protected:
    void connectLine( unsigned pin, DBit & bit ) {
        assert( pin > 0 );
        assert( pin <= line_count_ );

        line_[pin-1].assign( bit );
    }

    DBit bit( unsigned s ) {
        return s != 0 ? 1 : 0;
    }

    DBit line( unsigned pin ) {
        assert( pin > 0 );
        assert( pin <= line_count_ );

        return line_[pin-1].state();
    }

private:
    DLine * line_;
    unsigned line_count_;
};

inline void DLine::connectTo( DChip & chip, unsigned line ) {
    connectTo( chip.getLine(line) );
}


#endif // DCE_H_
