/*
    Space Invaders arcade machine emulator

    Copyright (c) 1997-2003,2004 Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef INVADERS_H_
#define INVADERS_H_

#include <math.h>

#include <emu/emu_math.h>
#include <emu/emu_memory_iostream.h>
#include <emu/emu_sample_player.h>
#include <emu/emu_standard_machine.h>
#include <cpu/i8080.h>
#include <sound/rcfilter.h>
#include <sound/sn76477.h>
#include <sound/waveform.h>

#include <ase/ase_lowpass_filter.h>
#include <ase/ase_noise.h>
#include <ase/ase_timer555_astable.h>

struct SpaceInvadersBoard : public I8080Environment
{
    SpaceInvadersBoard( unsigned char * vram );

    ~SpaceInvadersBoard();

    void run( unsigned samplesPerFrame );
    void reset();

    // Implementation of the I8080Environment interface
    unsigned char readByte( unsigned addr );
    void writeByte( unsigned, unsigned char );
    unsigned char readPort( unsigned port );
    void writePort( unsigned, unsigned char );

    unsigned now();

    // Member variables
    unsigned char   ram_[0x5000];
    unsigned char   port1i_;    // Port 1 in
    unsigned char   port2i_;    // Port 2 in
    unsigned char   port2o_;    // Port 2 out
    unsigned char   port3o_;    // Port 3 out
    unsigned char   port4lo_;   // Port 4 out (lo)
    unsigned char   port4hi_;   // Port 4 out (hi)
    unsigned char   port5o_;    // Port 5 out
    unsigned char * vram_;
    unsigned char   color_prom_[2*1024];
    I8080 *         cpu_;
    unsigned        samples_this_frame_;

    // Walk circuit
    ATimer555Astable    walk_timer_;
    ALowPassRCFilter    walk_filter1_;
    ALowPassRCFilter    walk_filter2_;

    // Flash circuit (base explosion)
    AWhiteNoise         flash_noise_;
    ALowPassRCFilter    flash_filter1_;
    ALowPassRCFilter    flash_filter2_;
};

/**
    Space Invaders machine driver.

    @author Alessandro Scotti
    @version 1.1
*/
class SpaceInvaders : public TStandardMachine
{
public:
    virtual ~SpaceInvaders() {
        delete main_board_;
    }

    virtual bool setResourceFile( int id, const unsigned char * buf, unsigned len );

    virtual void run( TFrame * frame, unsigned samplesPerFrame, unsigned samplingRate );

    virtual void reset();

    static TMachine * createInstance() {
        return new SpaceInvaders;
    }

protected:
    SpaceInvaders();

    bool initialize( TMachineDriverInfo * info );

    void setActivePalette( unsigned char palette );

    void renderAnalogSounds( TFrame * frame, unsigned samplesPerFrame, unsigned samplingRate );

    void repaintScreen();

    enum {
        ExtendPlayVibratoAmplitude = 70
    };

protected:
    unsigned            palette_data_[8][8];
    unsigned char       active_palette_;
    unsigned char       settings_;
    SpaceInvadersBoard *main_board_;
    TSamplePlayer       samples_[10];

    /* TODO
    T555Astable             extend_play_timer_;
    */
    TMixerBuffer            extend_play_sound_buffer_;
    TSquareWaveGenerator    extend_play_tone_;
    unsigned                extend_play_vibrato_offset_;

    SN76477                 ufo_sound_;
};

class SpaceAttackII : public SpaceInvaders
{
public:
    static TMachine * createInstance() {
        return new SpaceAttackII;
    }

protected:
    SpaceAttackII();

    bool initialize( TMachineDriverInfo * info );
};

class OzmaWars : public SpaceInvaders
{
public:
    static TMachine * createInstance() {
        return new OzmaWars;
    }

protected:
    OzmaWars();

    bool initialize( TMachineDriverInfo * info );
};

class SpaceInvadersPartII : public SpaceInvaders
{
public:
    static TMachine * createInstance() {
        return new SpaceInvadersPartII;
    }

protected:
    SpaceInvadersPartII();

    bool initialize( TMachineDriverInfo * info );
};

class LunarRescue : public SpaceInvaders
{
public:
    static TMachine * createInstance() {
        return new LunarRescue;
    }

protected:
    LunarRescue();

    bool initialize( TMachineDriverInfo * info );
};

#endif // INVADERS_H_
